@@ -2,7 +2,7 @@ package web;
 require 5.001;
 
 ##############################################################################
-# $Id: web.pm,v 1.30 2000/01/21 16:51:17 unrzc9 Exp $                 
+# $Id: web.pm,v 1.44 2002/09/12 09:38:13 unrzc9 Exp $                 
 # 
 # See the bottom of this file for the POD documentation.  Search for the
 # string '=head'.
@@ -23,9 +23,9 @@ require 5.001;
 # and Steve Brenner's cgi-lib.pl.
 #
 ##############################################################################
-# Last Modified on:	$Date: 2000/01/21 16:51:17 $
+# Last Modified on:	$Date: 2002/09/12 09:38:13 $
 # By:			$Author: unrzc9 $
-# Version:		$Revision: 1.30 $ 
+# Version:		$Revision: 1.44 $ 
 ##############################################################################
 
 use strict;
@@ -35,9 +35,9 @@ BEGIN {
     use vars       qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);
 
     # if using RCS/CVS, this may be preferred
-    $VERSION = do { my @r = (q$Revision: 1.30 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
-    $web::VERSION = do { my @r = (q$Revision: 1.30 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
-    $web::revision = '$Id: web.pm,v 1.30 2000/01/21 16:51:17 unrzc9 Exp $';
+    $VERSION = do { my @r = (q$Revision: 1.44 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+    $web::VERSION = do { my @r = (q$Revision: 1.44 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r };
+    $web::revision = '$Id: web.pm,v 1.44 2002/09/12 09:38:13 unrzc9 Exp $';
     # The above must be all one line, for MakeMaker
 
     @ISA         = qw(Exporter);
@@ -45,9 +45,9 @@ BEGIN {
     			&ReadLayout &Read_Parafile
     			&NUnlock &NLock &isZeit &isDatum &isURL &isMail &Fehlermeldung
     			&ReadParse &PrintHeader &Check_Name &numerically &Get_Seconds 
-    			&GetWeekDay &isLeapYear &GetYearDay &GetDatebyYDay 
-    			&Add_Days_to_Date &CgiDie &GetSentence 
-    			&NUnlockAll &RemoveHTML &Redirect);
+    			&GetWeekDay &isLeapYear &MakeTimeLocal &GetYearDay &GetDatebyYDay 
+    			&Add_Days_to_Date &CgiDie &GetSentence &GetPassedDaysbyMonth
+    			&NUnlockAll &RemoveHTML &Redirect &isIP);
     %EXPORT_TAGS = ( );     # eg: TAG => [ qw!name1 name2! ],
 
     # your exported package globals go here,
@@ -94,21 +94,28 @@ if (isLeapYear($web::jahr)) {
 }
 
 # Definitions for the date
-%web::US_MONATSNAME = ("Jan", 1, "Feb", 2, "Mar", 3, "Apr", 4, "May", 5, "Jun", 6, "Jul", 
-                   7, "Aug", 8, "Sep", 9, "Oct", 10, "Nov", 11, "Dec", 12);
+%web::US_MONATSNAME = (1, "Jan", 2, "Feb", 3, "Mar", 4, "Apr", 5, "May", 6, "Jun",  
+                    	7, "Jul", 8, "Aug", 9, "Sep", 10, "Oct", 11, "Nov", 12, "Dec");
 %web::GER_MONATSNAME = (1, "Januar", 2, "Februar", 3, "M&auml;rz", 4, "April", 5, "Mai", 
 	      6, "Juni", 7, "Juli", 8, "August", 9, "September", 10, "Oktober", 
 	      11, "November", 12, "Dezember");
 @web::wochentag = ("Sonntag","Montag","Dienstag","Mittwoch","Donnerstag","Freitag","Samstag");
+@web::weekdays = ("Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday");
 $web::schaltjahr = 2000;
 
-$web::zeit=$web::tag.'.'.$web::monat.'.'.$web::jahr.' - ';
-$web::zeit =$web::zeit.$web::stunde.':'.$web::minute.':'.$web::sek;
-$web::UHRZEIT ="$web::stunde:$web::minute:$web::sek";
-$web::datum = "$web::tag. $web::GER_MONATSNAME{$web::monat} $web::jahr";
-$web::sektime = $web::sek+$web::minute*60+$web::stunde*3600;
-$web::tagzeit = $web::tag+ 30* $web::monat+ 365* $web::jahr;
-$web::PARACOMMENT_SIGN = "_comment";
+$web::zeit		= $web::tag.'.'.$web::monat.'.'.$web::jahr.' - ';
+$web::zeit 		= $web::zeit.$web::stunde.':'.$web::minute.':'.$web::sek;
+$web::UHRZEIT 		= "$web::stunde:$web::minute:$web::sek";
+$web::datum 		= "$web::tag. $web::GER_MONATSNAME{$web::monat} $web::jahr";
+$web::datum_german 	= $web::datum;
+$web::datum_german_long = $web::wochentag[$web::wtag].", ".$web::datum_german;
+$web::datum_english 	= $web::tag.". ".$web::US_MONATSNAME{$web::monat}." ".$web::jahr;
+$web::datum_english_long = $web::weekdays[$web::wtag].", ".$web::datum_english;
+$web::sektime 		= $web::sek+$web::minute*60+$web::stunde*3600;
+$web::tagzeit           = GetPassedDaysbyMonth($web::monat);
+$web::tagzeit 		+= $web::tag;
+$web::tagzeit           += 365* $web::jahr;
+$web::PARACOMMENT_SIGN 	= "_comment";
 
 $web::LOCK_LOCATION = "/tmp";
 	# Where do we put our lockfiles
@@ -131,7 +138,7 @@ $web::allowuploads = 1;
 	# Do we allow file-uploads? 1=yes, 0=no
 $web::uploadfile = "/tmp/uploader.$$";
 	# Which file will be used temporary for fileuploads.
-my $READPARSE_DEBUG=0;
+$web::ReadParse_Debug =0;
 	# Set it to 1 for debugging
 %web::lockliste = ();
 	# This list will be filled and emptied by the locking
@@ -140,7 +147,8 @@ $web::errorlayout_file = "";
 	# Default Fehlerlayout-Datei fuer Fehlermeldungen. Kann von Programmen
 	# ueberschrieben werden, um beim Aufruf der Funktion Fehlermeldung() sich die
 	# Angabe der Layoutdatei zu ersparen.
-		
+$web::POST_MAX = 1024 * 100;
+	# Maximale Groesse fuer Uebertragungen in Bytes.		
 ##############################################################################
 # SubRoutines
 ##############################################################################
@@ -317,13 +325,13 @@ sub Read_Parafile {
     open(f1,"<$filename");
       while(<f1>) {
         chomp($_);
-        if ($_ =~ /^#/) {
+        if ($_ =~ /^\s*#/) {
            if ($lastcomment) {
               $kurznamebez = $lastcomment.$web::PARACOMMENT_SIGN;
               $resulthash{$kurznamebez} .= $';     
           }        
         } else {
-           ($entry_name, $entry_value) = split(/\t+/,$_,2);  
+           ($entry_name, $entry_value) = split(/\s+/,$_,2);  
            if ($entry_name) { 
              $resulthash{$entry_name} = $entry_value;
              $lastcomment = $entry_name;
@@ -343,6 +351,46 @@ sub Read_Parafile {
   return %resulthash;
 }
 ##############################################################################
+sub isIP {
+ my $ip = $_[0];
+
+ # This routine was mainly made by Rolf Rost (see perldoc below)
+ 
+ if (not ($ip =~ /\./)) { 
+   return 0;
+ }
+
+ if ( $ip =~ /[^0-9\.]/ ){ 
+   # An IP consists out of numbers and the dot only..
+   return 0;
+ } 
+ 
+ my $val = my($a, $b, $c, $d) = split(/\./, $ip);
+ # A valid IP consists out of 4 parts
+ if( $val != 4 ){ 
+   return 0;
+ }
+ 
+ # Check range:
+ if (  ($a <0 || $a >255) || ($b <0 || $b >255) || ($c<0 || $c >255) || ($d <0 || $d >255)  ){
+   return 0;
+ }
+ 
+ # All numbers at 0 is invalid too:
+ if ( ($a == 0) && ($b == 0) && ($c == 0) && ($d == 0)){
+   return 0;
+ }
+ 
+ # All numbers at 255 is invalid too:
+ if ( ($a == 255) && ($b == 255) && ($c == 255) && ($d== 255) ){
+   return 0;
+ }
+ 
+ # The syntax is valid
+ return 1;
+
+}
+##############################################################################
 sub isZeit {
   my ($testzeit) = @_;
   if (not $testzeit) {return 0;}
@@ -470,11 +518,11 @@ sub CgiDie {
 }
 ##############################################################################
 sub ReadParse {
- my ($buffer);
+ my $buffer;
  my ($namebuffer,$valuebuffer);
- my (@nvpairs);
- my ($pair);
- my ($boundary)="";
+ my @nvpairs;
+ my $pair;
+ my $boundary ="";
  my ($fieldname, $filetype);
  my ($filename);
  my ($savefile, $head);
@@ -484,14 +532,27 @@ sub ReadParse {
  my $bytes;
  my %in;
  my $tmpsavefile = $web::uploadfile;
+ my ($meth,$content_length) = ('','');
+ $content_length = defined($ENV{'CONTENT_LENGTH'}) ? $ENV{'CONTENT_LENGTH'} : 0;
+ $meth =$ENV{'REQUEST_METHOD'} if defined($ENV{'REQUEST_METHOD'});
+ my $type = $ENV{'CONTENT_TYPE'};
  
- if ($ENV{'CONTENT_TYPE'} =~ /multipart\//) {
+  # Disable warnings as this code deliberately uses local and environment
+  # variables which are preset to undef (i.e., not explicitly initialized)
+  my $perlwarn = $^W;
+  $^W = 0;
+ 
+ if (($web::POST_MAX > 0) && ($content_length > $web::POST_MAX)) {
+   $in{'status'} = "Content-Length zu gross";
+   return %in;
+ }
+ if (($ENV{'CONTENT_TYPE'}) && ($ENV{'CONTENT_TYPE'} =~ /multipart\//)) {
    if ($ENV{'CONTENT_TYPE'} =~ /boundary=(--\S+)/) {
      $boundary="--".$1;
    }
  }
  if ($boundary ne "") {
-   if ($READPARSE_DEBUG) {
+   if ($web::ReadParse_Debug) {
      print(&PrintHeader);
    }
    $head=1;
@@ -532,7 +593,7 @@ sub ReadParse {
        if ($savefile) {
          close FILE;
          $savefile=0;
-         if ($READPARSE_DEBUG) { 
+         if ($web::ReadParse_Debug) { 
            print "Datei $filename ($bytes Byte) wurde erfolgreich nach $tmpsavefile gespeichert.<BR>";
          }
          if ($web::allowuploads) {
@@ -562,7 +623,7 @@ sub ReadParse {
      } elsif ($filename eq "" && $fieldname ne "") {
        $in{$fieldname} .= $_; 
      }
-     if ($READPARSE_DEBUG && /^[\n\r\t\x20-\x7f\xa0-\xff]*$/) { 
+     if ($web::ReadParse_Debug && /^[\n\r\t\x20-\x7f\xa0-\xff]*$/) { 
        print "** $_ <br>";
      }
    }
@@ -572,29 +633,43 @@ sub ReadParse {
    }
    for $i (keys(%in)) {
      $in{$i} =~ s/[\n\r]+$//;
-     if ($READPARSE_DEBUG) {
+     if ($web::ReadParse_Debug) {
        print "in{$i} = (".$in{$i}.")<br>\n";
      }
    }
    unlink($tmpsavefile); 
    return (%in); 
  } else {
-  
-   if ($ENV{'REQUEST_METHOD'} eq "GET") { $buffer = $ENV{'QUERY_STRING'}; } 
+#   if (!defined $meth || $meth eq '' || $meth eq 'GET' || 
+#      $type eq 'application/x-www-form-urlencoded') {
+#     $buffer = $ENV{'QUERY_STRING'} if defined $ENV{'QUERY_STRING'};; 
+#     $buffer ||= $ENV{'REDIRECT_QUERY_STRING'} if defined $ENV{'REDIRECT_QUERY_STRING'};      
+#   } else {
+#  
+#     read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});  
+#   }
+   if ($ENV{'REQUEST_METHOD'} eq "GET") { $buffer = $ENV{'QUERY_STRING'} || $ENV{'REDIRECT_QUERY_STRING'}; } 
    else { read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});  }
- 
-   if (!($buffer)) {$buffer=substr($ENV{'PATH_INFO'},1,length($ENV{'PATH_INFO'}));}
-   if (!($buffer)) {$buffer=@ARGV; }
+
+   if ((not $buffer) && ($ENV{'PATH_INFO'})) {   
+     $buffer = $ENV{'PATH_INFO'};
+     $buffer =~ s/^\///;
+   } elsif (not $buffer) {
+     $buffer = @ARGV; 
+   }
  
    @nvpairs = split(/&/,$buffer);
    foreach $pair (@nvpairs)
     {  
       ($namebuffer, $valuebuffer) = split(/=/, $pair);
-      $namebuffer =~ tr/+/ /;
-      $namebuffer =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
-
-      $valuebuffer =~ tr/+/ /;
-      $valuebuffer =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
+      if ($namebuffer) {
+        $namebuffer =~ tr/+/ /;
+        $namebuffer =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
+      }
+      if ($valuebuffer) {
+        $valuebuffer =~ tr/+/ /;
+        $valuebuffer =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
+      }
       if (exists($in{$namebuffer})) {
         $in{$namebuffer} .= ", $valuebuffer";
       } else {
@@ -606,10 +681,54 @@ sub ReadParse {
 } 
 ################################################################################# 
 sub PrintHeader {
-  if (not $web::HEADER) {
-    $web::HEADER = 1;
-    return "Content-type: text/html\n\n"; 
+  my $reset_cookie = shift;
+  my $reprint = shift || $web::HEADER;
+  my $cookies = shift;
+  my $path = shift || "/";
+  my $expire = shift || 30;
+  my $domain = shift;
+  my $key;
+  my $result;
+  my $SETDOMAIN;
+  
+  if ($reprint==1) {
+    return;
   }
+  if ($domain) {
+    $SETDOMAIN = " domain=$domain;";
+  }
+  $web::HEADER = 1;
+  if ($reset_cookie) {
+    my ($sec,$min,$hr,$mday,$mon,$yr,$wday,$yday,$isdst) = gmtime(time + (86400*$expire)); 
+    my @days = ("Sun","Mon","Tue","Wed","Thu","Fri","Sat");
+    my @months = ("Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec");
+    my $expdate = sprintf("%3s, %02d-%3s-%4d %02d:%02d:%02d GMT",$days[$wday],$mday,$months[$mon],$yr+1900,$hr,$min,$sec);
+    
+    if (ref($cookies) eq 'HASH') {
+      foreach $key (keys %{$cookies}) {
+        $result .= "Set-Cookie: $key=$cookies->{$key};";
+        $result .= " expires=$expdate;";
+        $result .= " path=$path;";
+        $result .= "$SETDOMAIN"; 
+        $result .= "\n"; 
+      }
+    } elsif ($ENV{'HTTP_COOKIE'}) {
+      my ($name,$cid);
+      my $i;
+      my @cookiefeld = split(/;/,$ENV{'HTTP_COOKIE'});
+      foreach $i (@cookiefeld) {
+        ($name,$cid) = split(/=/,$i);
+        $result .= "Set-Cookie: $name=$cid;";
+        $result .= " expires=$expdate;";
+        $result .= " path=$path;";
+        $result .= "$SETDOMAIN";
+        $result .= "\n";
+        $web::COOKIES{$name} = $cid;
+      }
+    }  
+  }
+  $result .= "Content-type: text/html\n\n"; 
+  return $result;
 }
 ##############################################################################
 sub Check_Name {
@@ -632,69 +751,41 @@ sub Get_Seconds {
 }
 ################################################################################
 sub isLeapYear {
-  my ($thisyear) = $_[0] || $web::jahr;
-  my $count = $web::schaltjahr;
-  my $hcount = $web::schaltjahr;
-  my $fhcount = $web::schaltjahr;
-  
-  # I dont use the math-modules to avoid loading too
-  # much moduls, that would simply do the same...
-  
-  if (not $thisyear) {
+    my $year  = shift || $web::jahr;
+
+    return 1
+        if ( ( $year / 4   == int( $year / 4   ) ) &&
+             ( $year / 100 == int( $year / 100 ) ) &&
+             ( $year / 400 == int( $year / 400 ) )    );
+
+    return 1
+        if ( ( $year / 4   == int( $year / 4   ) ) &&
+             ( $year / 100 != int( $year / 100 ) )    );
+
     return 0;
-  }
-  if ($thisyear > $count) {
-    while($thisyear > $count) {
-      $count += 4;
-    }
-    while($thisyear > $hcount) {
-      $hcount += 100;
-    }
-    while($thisyear > $fhcount) {
-      $fhcount += 400;
-    }
-  } else {
-    while($thisyear < $count) {
-      $count -= 4;
-    }
-    while($thisyear < $hcount) {
-      $hcount -= 100;
-    }
-    while($thisyear < $fhcount) {
-      $fhcount -= 400;
-    }
-  }
-  if ($count==$thisyear) {
-    if ($hcount==$thisyear) {
-      if ($fhcount==$thisyear) {
-        return 1;
-      } else {
-        return 0;
-      }
-    } else {
-      return 1;
-    }
-  } else {
-    return 0;
-  }
 }
 ################################################################################
 sub GetDatebyYDay {
   my ($yday) = $_[0];
+  my $jahr = $_[1] || $web::jahr;
   my $msumme;
   my $mon;
   my $this_monat;
   my $schalt;
+  my $rescue;
   
+  $rescue = $web::tage_im_monat[1];
+  $schalt = isLeapYear($jahr);  
+  $web::tage_im_monat[1] = 28 + $schalt;
   
-  $schalt = isLeapYear($web::jahr);  
+ 
   
   if (($yday < 1) || ($yday > (365+$schalt))) {
     return;
   }
   for ($mon=0; $mon<=11; $mon++) {
     $msumme += $web::tage_im_monat[$mon];
-    if ($msumme > $yday) {
+    if ($msumme >= $yday) {
       $this_monat = $mon+1;
       $msumme -= $web::tage_im_monat[$mon];
       $yday -= $msumme;
@@ -705,48 +796,90 @@ sub GetDatebyYDay {
       last;
     }
   }
-
+  $web::tage_im_monat[1] = $rescue;
   return "$yday.$this_monat";
 }
 ################################################################################
 sub GetWeekDay {
- my ($inputdatum)=$_[0];
- my ($gmday, $gmonat, $gjahr);
- my ($gsec, $gmin, $gstd, $gwday, $gyday, $gisday);
- my $gwdtime;
+    my $inputdatum = shift;
 
-use Time::Local;
- 
- # Das Datum muss das Format DD.MM.YYYY haben.
- # Als Ergebnis erhaelt man den Wochentag. 0=Sonntag, 6=Samstag
- # Im Fehlerfalle wird -1 zurueckgegeben.
- 
- 
- if (not isDatum($inputdatum)) {return "-1";}
- ($gmday, $gmonat, $gjahr) =split(/\./,$inputdatum);
- $gmonat--;
- $gwdtime = timelocal(1,1,1, $gmday, $gmonat, $gjahr);
- ($gsec, $gmin, $gstd, $gmday, $gmonat, $gjahr, $gwday, $gyday, $gisday) = localtime($gwdtime);
+    return '-1'
+        unless ( isDatum( $inputdatum ) );
 
- return $gwday;
+    my ( $mday, $mon, $year ) = split( /\./, $inputdatum, 3 );
+
+    $year += 1900
+        if ( $year < 999 );
+
+    my $a = int( ( 14 - $mon ) / 12 );
+    my $y = $year - $a;
+    my $m = $mon + ( 12 * $a ) - 2;
+    my $result = ( $mday + $y + int($y/4) - int($y/100) + int($y/400) + int(31/(12*$m)) ) % 7;
+    return $result;
+}
+##############################################################################
+sub MakeTimeLocal {
+  my $zeit = shift;
+  my ($dat_datum,$dat_uhr);
+  my ($dat_tag,$dat_monat,$dat_jahr);
+  my ($stunds, $mins, $seks);
+  my $result;
+  
+  if (not $zeit) {
+    return;
+  }
+  use Time::Local;
+  ($dat_datum,$dat_uhr) = split(/ - /,$zeit,2);
+  ($dat_tag, $dat_monat, $dat_jahr) = split(/[\.:]/,$dat_datum,3);
+  ($stunds, $mins, $seks) = split(/:/,$dat_uhr,3);  
+  $dat_monat--;
+  $dat_jahr -= 1900;
+
+  if ($dat_monat <0) { $dat_monat=0;}
+  $result = timelocal($seks, $mins, $stunds, $dat_tag, $dat_monat, $dat_jahr);
+  return $result;
 }
 ################################################################################
 sub GetYearDay {
- my ($inputdatum)=$_[0];
- my ($gmday, $gmonat, $gjahr); 
- my ($gsec, $gmin, $gstd, $gwday, $gyday, $gisday);
- my $gwdtime;
+  my $inputdatum = shift;
+  my $res  = 0;
+  my $cnt;
+  my $save= $web::tage_im_monat[1];
+
+  return '-1'
+        unless ( isDatum( $inputdatum ) );
+
+  my ( $mday, $mon, $year ) = split( /\./, $inputdatum, 3 );
+
+  $year += 1900  if ( $year < 999 );
+  $mon--;
+
+  $web::tage_im_monat[1] = 28 + isLeapYear( $year );
+
+  for ( $cnt = 0; $cnt < $mon; $cnt++ ) {
+      $res += $web::tage_im_monat[$cnt];
+  }
+
+  $web::tage_im_monat[1] = $save;
+
+  return ( $res + $mday );
   
-  use Time::Local;
-  if (not isDatum($inputdatum)) {return "-1";}
- 
-  ($gmday, $gmonat, $gjahr) =split(/\./,$inputdatum);
-  $gmonat--;
-  $gwdtime = timelocal(1,1,1, $gmday, $gmonat, $gjahr);
-  ($gsec, $gmin, $gstd, $gmday, $gmonat, $gjahr, $gwday, $gyday, $gisday) = localtime($gwdtime);
-  $gyday++;
+}
+################################################################################
+sub GetPassedDaysbyMonth {
+  my $monat = shift;
+  my $i;
+  my $count;
   
- return $gyday;
+  if (($monat<=1) || ($monat > 12)) {
+    return 0;
+  }
+  $monat--;
+  
+  for ($i=0; $i<$monat; $i++) {
+    $count += $web::tage_im_monat[$i];
+  }
+  return $count;
 }
 ################################################################################
 sub Add_Days_to_Date {
@@ -770,14 +903,21 @@ sub Add_Days_to_Date {
   $schalt = isLeapYear($thisyear);  
   
   $tagzahl += $inputtage;
-  while ($tagzahl > (365 + $schalt)) {
-    $tagzahl =  $tagzahl - (365 + $schalt);
-    $thisyear++;
-    $schalt = isLeapYear($thisyear); 
+  
+  if ($tagzahl > 0) {
+    while ($tagzahl > (365 + $schalt)) {
+      $tagzahl =  $tagzahl - (365 + $schalt);
+      $thisyear++;
+      $schalt = isLeapYear($thisyear); 
+    }
+  } else {
+    while ($tagzahl <=0) {
+      $thisyear--;
+      $schalt = isLeapYear($thisyear); 
+      $tagzahl += (365 + $schalt);
+    } 
   }
-  $web::jahr = $thisyear;
-  my $resultdate = GetDatebyYDay($tagzahl);
-  $web::jahr = $rescue;
+  my $resultdate = GetDatebyYDay($tagzahl,$thisyear);
   $resultdate = "$resultdate.$thisyear";
   return $resultdate;
 }
@@ -979,6 +1119,11 @@ In using this command, you can remove all file-locks, that was set with
 NLock() and which wasn't removed before.
 It takes the list of file-locks out of the hash %web::lockliste.
 
+=head2 MakeTimeLocal
+
+Translates the timesyntax of web.pm into the standard time-syntax, which
+is the seconds since 1970.
+
 =head2 GetYearDay
 
 This routine can be used to calculate a day's position within
@@ -1062,6 +1207,19 @@ Example:
 	print "Time differs with $diff_sekunden seconds.\n";
 
 
+=head2 GetPassedDaysbyMonth
+
+Returns the number of days that have passed by the inputvalue, which
+represend the number of a month.
+
+Example:
+
+	$month = 8;
+	$passed_days = GetPassedDaysbyMonth($month);
+	print "$passed_days days have passed, before the $month. month came\n";
+
+
+
 =head2 Check_Name
 
 Sometimes you need to get file names or other data from the internet.
@@ -1097,6 +1255,27 @@ won't work.
 =head2 PrintHeader
 
 Returns the string "Content-type: text/html\n\n", if it wasn't returned before.
+Additionally it can reset cookies if there were some before on this domain
+or set new cookies by using a hash-reference.
+The additional parameters are:
+
+Arguments:
+
+	   1		Activates the setting of cookies. If new cookies
+	   		are defined by argument 3, these will be set. Otherwise
+	   		the cookies as given by $ENV{'HTTP_COOKIES'} are
+	   		used.
+	   2		On default, PrintHeader() will return nothing, if it
+	   		was called before. By setting this argument unlike 1
+	   		it will ignore previous calls.
+	   3		A hash-reference, which defines the cookies to be set.
+	   4		The path-value for the cookies. On default its set to
+	   		"/".
+	   5		The lifetime-value for cookies in days. On default its
+	   		set to 30.
+
+
+
 
 =head2 ReadParse
 
@@ -1178,6 +1357,12 @@ Checks if the argument is a valid German date. The syntax for a date
 was set to: DD.MM.YYYY.
 
 
+=head2 isIP
+
+Checks the given argument for a valid IP-syntax. It returns TRUE
+on success. (This routine was invoked by Rolf Rost, http://www.-i-netlab.de)
+
+
 =head2 isZeit
 
 Checks if the argument is a valid time. The syntax of a time
@@ -1335,6 +1520,8 @@ Thanks very much to:
 
 =item Rolf Rost (rolfrost@yahoo.com)
 
+=item Harald Mattern (webmaster@tsmweb.de)
+
 =cut
 
 
